@@ -7,11 +7,7 @@ import websockets
 import asyncio
 import sys
 import re
-from asyncio import Queue, CancelledError
-if sys.version_info < (3, 5):
-    from asyncio import async as ensure_future
-else:
-    from asyncio import ensure_future
+from asyncio import CancelledError
 from collections import deque
 
 from functools import partial
@@ -29,6 +25,11 @@ from hbmqtt.adapters import (
     WebSocketsWriter)
 from .plugins.manager import PluginManager, BaseContext
 
+if sys.version_info < (3, 5):
+    from asyncio import async as ensure_future
+else:
+    from asyncio import ensure_future
+
 _defaults = {
     'timeout-disconnect-delay': 2,
     'auth': {
@@ -53,6 +54,9 @@ class BrokerException(BaseException):
 
 
 class RetainedApplicationMessage:
+
+    __slots__ = ('source_session', 'topic', 'data', 'qos')
+
     def __init__(self, source_session, topic, data, qos=None):
         self.source_session = source_session
         self.topic = topic
@@ -84,10 +88,10 @@ class Server:
         self.conn_count += 1
         if self.max_connections > 0:
             self.logger.info("Listener '%s': %d/%d connections acquired" %
-                              (self.listener_name, self.conn_count, self.max_connections))
+                             (self.listener_name, self.conn_count, self.max_connections))
         else:
             self.logger.info("Listener '%s': %d connections acquired" %
-                              (self.listener_name, self.conn_count))
+                             (self.listener_name, self.conn_count))
 
     def release_connection(self):
         if self.semaphore:
@@ -95,10 +99,10 @@ class Server:
         self.conn_count -= 1
         if self.max_connections > 0:
             self.logger.info("Listener '%s': %d/%d connections acquired" %
-                              (self.listener_name, self.conn_count, self.max_connections))
+                             (self.listener_name, self.conn_count, self.max_connections))
         else:
             self.logger.info("Listener '%s': %d connections acquired" %
-                              (self.listener_name, self.conn_count))
+                             (self.listener_name, self.conn_count))
 
     @asyncio.coroutine
     def close_instance(self):
@@ -216,9 +220,10 @@ class Broker:
             self._retained_messages = dict()
             self.transitions.start()
             self.logger.debug("Broker starting")
-        except MachineError as me:
-            self.logger.warn("[WARN-0001] Invalid method call at this moment: %s" % me)
-            raise BrokerException("Broker instance can't be started: %s" % me)
+        except (MachineError, ValueError) as exc:
+            # Backwards compat: MachineError is raised by transitions < 0.5.0.
+            self.logger.warning("[WARN-0001] Invalid method call at this moment: %s" % exc)
+            raise BrokerException("Broker instance can't be started: %s" % exc)
 
         yield from self.plugins_manager.fire_event(EVENT_BROKER_PRE_START)
         try:
@@ -302,9 +307,10 @@ class Broker:
             self._subscriptions = dict()
             self._retained_messages = dict()
             self.transitions.shutdown()
-        except MachineError as me:
-            self.logger.debug("Invalid method call at this moment: %s" % me)
-            raise BrokerException("Broker instance can't be stopped: %s" % me)
+        except (MachineError, ValueError) as exc:
+            # Backwards compat: MachineError is raised by transitions < 0.5.0.
+            self.logger.debug("Invalid method call at this moment: %s" % exc)
+            raise BrokerException("Broker instance can't be stopped: %s" % exc)
 
         # Fire broker_shutdown event to plugins
         yield from self.plugins_manager.fire_event(EVENT_BROKER_PRE_SHUTDOWN)
@@ -350,8 +356,8 @@ class Broker:
         try:
             handler, client_session = yield from BrokerProtocolHandler.init_from_connect(reader, writer, self.plugins_manager, loop=self._loop)
         except HBMQTTException as exc:
-            self.logger.warn("[MQTT-3.1.0-1] %s: Can't read first packet an CONNECT: %s" %
-                             (format_client_message(address=remote_address, port=remote_port), exc))
+            self.logger.warning("[MQTT-3.1.0-1] %s: Can't read first packet an CONNECT: %s" %
+                                (format_client_message(address=remote_address, port=remote_port), exc))
             #yield from writer.close()
             self.logger.debug("Connection closed")
             return
@@ -373,7 +379,7 @@ class Broker:
             # Get session from cache
             if client_session.client_id in self._sessions:
                 self.logger.debug("Found old session %s" % repr(self._sessions[client_session.client_id]))
-                (client_session,h) = self._sessions[client_session.client_id]
+                (client_session, h) = self._sessions[client_session.client_id]
                 client_session.parent = 1
             else:
                 client_session.parent = 0
@@ -393,7 +399,8 @@ class Broker:
             try:
                 client_session.transitions.connect()
                 break
-            except MachineError:
+            except (MachineError, ValueError):
+                # Backwards compat: MachineError is raised by transitions < 0.5.0.
                 self.logger.warning("Client %s is reconnecting too quickly, make it wait" % client_session.client_id)
                 # Wait a bit may be client is reconnecting too fast
                 yield from asyncio.sleep(1, loop=self._loop)
@@ -474,10 +481,10 @@ class Broker:
                         self.logger.debug("%s handling message delivery" % client_session.client_id)
                     app_message = wait_deliver.result()
                     if not app_message.topic:
-                        self.logger.warn("[MQTT-4.7.3-1] - %s invalid TOPIC sent in PUBLISH message, closing connection" % client_session.client_id)
+                        self.logger.warning("[MQTT-4.7.3-1] - %s invalid TOPIC sent in PUBLISH message, closing connection" % client_session.client_id)
                         break
                     if "#" in app_message.topic or "+" in app_message.topic:
-                        self.logger.warn("[MQTT-3.3.2-2] - %s invalid TOPIC sent in PUBLISH message, closing connection" % client_session.client_id)
+                        self.logger.warning("[MQTT-3.3.2-2] - %s invalid TOPIC sent in PUBLISH message, closing connection" % client_session.client_id)
                         break
                     yield from self.plugins_manager.fire_event(EVENT_BROKER_MESSAGE_RECEIVED,
                                                                client_id=client_session.client_id,
@@ -497,7 +504,6 @@ class Broker:
         self.logger.debug("%s Client disconnected" % client_session.client_id)
         server.release_connection()
 
-
     def _init_handler(self, session, reader, writer):
         """
         Create a BrokerProtocolHandler and attach to a session
@@ -565,8 +571,6 @@ class Broker:
                 del self._retained_messages[topic_name]
 
     def add_subscription(self, subscription, session):
-        import re
-        wildcard_pattern = re.compile('.*?/?\+/?.*?')
         try:
             a_filter = subscription[0]
             if '#' in a_filter and not a_filter.endswith('#'):
@@ -584,7 +588,7 @@ class Broker:
             if a_filter not in self._subscriptions:
                 self._subscriptions[a_filter] = []
             already_subscribed = next(
-                (s for (s,qos) in self._subscriptions[a_filter] if s.client_id == session.client_id), None)
+                (s for (s, qos) in self._subscriptions[a_filter] if s.client_id == session.client_id), None)
             if not already_subscribed:
                 self._subscriptions[a_filter].append((session, qos))
             else:
