@@ -2,6 +2,7 @@
 #
 # See the file license.txt for copying permission.
 
+import asyncio
 import logging
 import ssl
 from urllib.parse import urlparse, urlunparse
@@ -9,13 +10,12 @@ from functools import wraps
 
 from hbmqtt.utils import not_in_dict_or_none
 from hbmqtt.session import Session
-from hbmqtt.mqtt.connack import *
-from hbmqtt.mqtt.connect import *
+from hbmqtt.mqtt.connack import CONNECTION_ACCEPTED
 from hbmqtt.mqtt.protocol.client_handler import ClientProtocolHandler
 from hbmqtt.adapters import StreamReaderAdapter, StreamWriterAdapter, WebSocketsReader, WebSocketsWriter
 from hbmqtt.plugins.manager import PluginManager, BaseContext
-from hbmqtt.mqtt.protocol.handler import EVENT_MQTT_PACKET_SENT, EVENT_MQTT_PACKET_RECEIVED, ProtocolHandlerException
-from hbmqtt.mqtt.constants import *
+from hbmqtt.mqtt.protocol.handler import ProtocolHandlerException
+from hbmqtt.mqtt.constants import QOS_0, QOS_1, QOS_2
 import websockets
 from websockets.uri import InvalidURI
 from websockets.handshake import InvalidHandshake
@@ -54,6 +54,7 @@ class ClientContext(BaseContext):
         super().__init__()
         self.config = None
 
+
 base_logger = logging.getLogger(__name__)
 
 
@@ -112,7 +113,6 @@ class MQTTClient:
         self.plugins_manager = PluginManager('hbmqtt.client.plugins', context)
         self.client_tasks = deque()
 
-
     @asyncio.coroutine
     def connect(self,
                 uri=None,
@@ -167,7 +167,7 @@ class MQTTClient:
             yield from self._handler.stop()
             self.session.transitions.disconnect()
         else:
-            self.logger.warn("Client session is not currently connected, ignoring call")
+            self.logger.warning("Client session is not currently connected, ignoring call")
 
     @asyncio.coroutine
     def reconnect(self, cleansession=None):
@@ -185,7 +185,7 @@ class MQTTClient:
         """
 
         if self.session.transitions.is_connected():
-            self.logger.warn("Client already connected")
+            self.logger.warning("Client already connected")
             return CONNECTION_ACCEPTED
 
         if cleansession:
@@ -210,7 +210,6 @@ class MQTTClient:
                 yield from asyncio.sleep(delay, loop=self._loop)
                 nb_attempt += 1
 
-
     @asyncio.coroutine
     def _do_connect(self):
         return_code = yield from self._connect_coro()
@@ -231,8 +230,8 @@ class MQTTClient:
         if self.session.transitions.is_connected():
             yield from self._handler.mqtt_ping()
         else:
-            self.logger.warn("MQTT PING request incompatible with current session state '%s'" %
-                             self.session.transitions.state)
+            self.logger.warning("MQTT PING request incompatible with current session state '%s'" %
+                                self.session.transitions.state)
 
     @mqtt_connected
     @asyncio.coroutine
@@ -310,7 +309,7 @@ class MQTTClient:
             Example of ``topics`` argument expected structure:
             ::
 
-                ['$SYS/broker/uptime', QOS_1), '$SYS/broker/load/#', QOS_2]
+                ['$SYS/broker/uptime', '$SYS/broker/load/#']
         """
         yield from self._handler.mqtt_unsubscribe(topics, self.session.next_packet_id)
 
@@ -332,6 +331,9 @@ class MQTTClient:
         self.logger.debug("Waiting message delivery")
         done, pending = yield from asyncio.wait([deliver_task], loop=self._loop, return_when=asyncio.FIRST_EXCEPTION, timeout=timeout)
         if deliver_task in done:
+            if deliver_task.exception() is not None:
+                # deliver_task raised an exception, pass it on to our caller
+                raise deliver_task.exception()
             self.client_tasks.pop()
             return deliver_task.result()
         else:
@@ -346,7 +348,7 @@ class MQTTClient:
         # Decode URI attributes
         uri_attributes = urlparse(self.session.broker_uri)
         scheme = uri_attributes.scheme
-        secure = True if scheme in ('mqtts', 'wss') else False        
+        secure = True if scheme in ('mqtts', 'wss') else False
         self.session.username = uri_attributes.username
         self.session.password = uri_attributes.password
         self.session.remote_address = uri_attributes.hostname
@@ -365,9 +367,6 @@ class MQTTClient:
         self._handler = ClientProtocolHandler(self.plugins_manager, loop=self._loop)
 
         if secure:
-            if self.session.cafile is None or self.session.cafile == '':
-                self.logger.warn("TLS connection can't be estabilshed, no certificate file (.cert) given")
-                raise ClientException("TLS connection can't be estabilshed, no certificate file (.cert) given")
             sc = ssl.create_default_context(
                 ssl.Purpose.SERVER_AUTH,
                 cafile=self.session.cafile,
@@ -416,22 +415,22 @@ class MQTTClient:
                 self.logger.debug("connected to %s:%s" % (self.session.remote_address, self.session.remote_port))
             return return_code
         except InvalidURI as iuri:
-            self.logger.warn("connection failed: invalid URI '%s'" % self.session.broker_uri)
+            self.logger.warning("connection failed: invalid URI '%s'" % self.session.broker_uri)
             self.session.transitions.disconnect()
             raise ConnectException("connection failed: invalid URI '%s'" % self.session.broker_uri, iuri)
         except InvalidHandshake as ihs:
-            self.logger.warn("connection failed: invalid websocket handshake")
+            self.logger.warning("connection failed: invalid websocket handshake")
             self.session.transitions.disconnect()
             raise ConnectException("connection failed: invalid websocket handshake", ihs)
         except (ProtocolHandlerException, ConnectionError, OSError) as e:
-            self.logger.warn("MQTT connection failed: %r" % e)
+            self.logger.warning("MQTT connection failed: %r" % e)
             self.session.transitions.disconnect()
             raise ConnectException(e)
 
     @asyncio.coroutine
     def handle_connection_close(self):
 
-        def cancel_tasks(self):
+        def cancel_tasks():
             while self.client_tasks:
                 task = self.client_tasks.popleft()
                 if not task.done():
@@ -457,7 +456,7 @@ class MQTTClient:
                 yield from self.reconnect()
             except ConnectException:
                 # Cancel client pending tasks
-                cancel_tasks(self)
+                cancel_tasks()
         else:
             # Cancel client pending tasks
             cancel_tasks()
